@@ -65,10 +65,14 @@ BayonneBinder("ivrscript1")
                         (Check)&Checks::chkStop},
 		{"start", false, (Method)&Methods::scrStart,
 			(Check)&Checks::chkStart},
-		{"connect", false, (Method)&Methods::scrStart,
+		{"connect", false, (Method)&Methods::scrConnect,
 			(Check)&Checks::chkConnect},
 		{"join", false, (Method)&Methods::scrJoin,
 			(Check)&Checks::chkJoin},
+		{"recall", false, (Method)&Methods::scrRecall,
+			(Check)&Checks::chkRecall},
+		{"release", true, (Method)&Methods::scrRelease,
+			(Check)&Checks::chkRecall},
 		{"reconnect", false, (Method)&Methods::scrReconnect,
 			(Check)&Checks::chkReconnect},
                 {NULL, false, NULL, NULL}}; 
