@@ -25,8 +25,13 @@ const char *Checks::chkJoin(Line *line, ScriptImage *img)
 	if(line->argc)
 		return "join uses no keywords or arguments";
 
-	if(getMember(line))
-		return "join has no members";
+	return NULL;
+}
+
+const char *Checks::chkRecall(Line *line, ScriptImage *img)
+{
+	if(line->argc)
+		return "recall uses no keywords or arguments";
 
 	return NULL;
 }
