@@ -104,12 +104,16 @@ public:
 	const char *chkDial(Line *line, ScriptImage *img);
 	const char *chkEndinput(Line *line, ScriptImage *img);
 	const char *chkReconnect(Line *line, ScriptImage *img);
+	const char *chkRecall(Line *line, ScriptImage *img);
 };
 
 class Methods : public BayonneSession
 {
 public:
 	bool scrKey(void);
+	bool scrRelease(void);
+	bool scrRecall(void);
+	bool scrConnect(void);
 	bool scrStart(void);
 	bool scrStop(void);
 	bool scrJoin(void);
