@@ -72,6 +72,7 @@ Bayonne::statetab Bayonne::states[] = {
 	{"collect", &BayonneSession::stateCollect, 'c'},
 	{"dial", &BayonneSession::stateDial, 'd'},
 	{"xfer", &BayonneSession::stateXfer, 'd'},
+	{"refer", &BayonneSession::stateRefer, 'w'},
 	{"hold", &BayonneSession::stateHold, 'd'},
 	{"recall", &BayonneSession::stateRecall, 'd'},
 	{"tone", &BayonneSession::stateTone, 't'},
@@ -93,6 +94,44 @@ Bayonne::statetab Bayonne::states[] = {
 	{"final", &BayonneSession::stateFinal, ' '},
 	{NULL, NULL, ' '}};
 
+Bayonne::Call::Call(BayonneSession *dest, const char *from)
+{
+	time_t buf;
+	const char *dn = dest->getSymbol("session.dialed");
+	if(!dn)
+		dn = "unknown";
+
+	strcpy(call_id, dest->var_sid);
+	setString(call_from, sizeof(call_from), from);
+	setString(call_to, sizeof(call_to), dn);
+	call_connect = 0;
+	refcount = 1;
+	type = dest->type;
+	call_start = time(&buf);
+}
+
+void Bayonne::Call::detach(const char *reason)
+{
+	if(reason && !reason[0])
+		setString(call_reason, sizeof(call_reason), reason);
+
+	--refcount;
+	if(!refcount)
+		BayonneBinder::reportCall(this);
+}
+
+void Bayonne::Call::attach(void)
+{
+	++refcount;
+}
+
+void Bayonne::Call::start(void)
+{
+	time_t buf;
+	if(!call_connect)
+		call_connect = time(&buf);
+}
+
 Bayonne::Traffic::Traffic()
 {
 	time_t ts;
