@@ -191,6 +191,7 @@ public:
 		STATE_COLLECT,
 		STATE_DIAL,
 		STATE_XFER,
+		STATE_REFER,
 		STATE_HOLD,
 		STATE_RECALL,
 		STATE_TONE,
@@ -304,6 +305,7 @@ public:
 		START_FORWARDED,	// start a forwarded call
 		START_RINGING,		// start a ringing call
 		START_HUNTING,		// start a hunting call
+		START_REFER,		// start a refer session
 		STOP_SCRIPT,
 		STOP_DISCONNECT,
 		STOP_PARENT,
@@ -322,6 +324,7 @@ public:
 
 		START_SCRIPT = START_INCOMING,
 		START_SELECTED = START_OUTGOING,
+		START_TRANSFER = START_REFER,
 
 		// libexec specific events
 
@@ -442,6 +445,7 @@ public:
 		AUDIO_DISCONNECT,	/* audio holding re-invite */
 		PEER_RECONNECT,		/* peer re-invite notice */
 		PEER_DISCONNECT,	/* peer holding notice */
+		PEER_REFER,		/* referred by peer */
 		DTMF_GENTONE = DTMF_GENUP,
 
 		// make modes and special timeslot stuff
@@ -465,6 +469,9 @@ public:
 		START_RELOCATE,		// start timeslot for relocation
 		STREAM_ACTIVE,
 		STREAM_PASSIVE,
+		JOIN_RECALL,
+		DROP_RECALL,
+		DROP_REFER,
 
 		// master control events
 
@@ -595,6 +602,7 @@ public:
 				timeout_t framing;
 			}	reconnect;
 
+			const char *dialing;
 			const char *name;
 			const char *errmsg;
 			BayonneSession *pid;
@@ -604,6 +612,26 @@ public:
 
 	}	Event;
 
+	/**
+ 	 * This is an internal call class.
+	 */
+	class __EXPORT Call
+	{
+	private:
+		volatile unsigned refcount;
+
+	public:
+		volatile time_t call_start, call_connect;
+		char call_from[65], call_to[65], call_reason[16];
+		char call_code[16], call_id[16];
+		calltype_t type;
+
+		Call(BayonneSession *target, const char *from);
+		void detach(const char *reason = NULL);
+		void attach(void);
+		void start(void);
+	};
+
 	/**
 	 * This is a class used for collecting statistics for call 
 	 * traffic measurement, such as might be used by MRTG.
@@ -664,6 +692,7 @@ public:
 		result_t result;
 		int pid;
 		libaudio_t *libaudio;
+		bool refer;
 
 		union
 		{
@@ -711,10 +740,7 @@ public:
 			{
 				const char *sequence;
 				bool flashing;
-				bool dialing;
-				bool exiting;
-				bool hangup;
-				bool dtmf;
+				bool dialing, exiting, hangup, dtmf, refer;
 				char *syncdigit;
 				timeout_t synctimer;
 				timeout_t duration;
@@ -735,12 +761,11 @@ public:
 			{
 				const char *dial;
 				const char *exit;
-				bool dtmf, drop, hangup;
+				bool dtmf, drop, hangup, refer;
 				BayonneSession *peer;
 				timeout_t answer_timer, hunt_timer;
 				Line *select;
 				unsigned index;
-				time_t start;
 				char digits[64];
 			}	join;
 
@@ -1975,7 +2000,7 @@ private:
 protected:
 	virtual const char *submit(const char **data);
 
-	virtual void billing(BayonneSession *p, BayonneSession *c, time_t start);
+	virtual void report(Call *call);
 
 	virtual ScriptCompiler *compiler(void);
 
@@ -1988,11 +2013,11 @@ protected:
 	BayonneBinder(const char *id);
 
 public:
+	static void reportCall(Call *call);
+
 	static const char *submitRequest(const char **data);
 
 	static ScriptCompiler *getCompiler(void);
-
-	static void reportBilling(BayonneSession *p, BayonneSession *c, time_t start);
 };
 
 /**
@@ -2009,6 +2034,7 @@ private:
 	friend class __EXPORT BayonneTranslator;
 	friend class __EXPORT BayonneDriver;
 	friend class __EXPORT Bayonne;
+	friend class __EXPORT Bayonne::Call;
 
 	BayonneSession() {};
 
@@ -2021,6 +2047,7 @@ protected:
 	static Mutex globalLock;	
 
 	std::ostream *logevents, *logtrace;
+	Call *call;
 	BayonneDriver *driver;
 	BayonneMsgport *msgport;
 	BayonneSession *peer;
@@ -2029,7 +2056,7 @@ protected:
 	uint8 seq;
 	uint16 evseq;
 	uint32 tseq;
-	bool offhook, dtmf, answered, starting, holding, connecting;
+	bool offhook, dtmf, answered, starting, holding, connecting, referring;
 	time_t audiotimer, starttime;
 	interface_t iface;
 	bridge_t bridge;
@@ -2099,6 +2126,7 @@ protected:
 	virtual bool enterBusy(Event *event);
 	virtual bool enterStandby(Event *event);
 	virtual bool enterXfer(Event *event);
+	virtual bool enterRefer(Event *event);
 	virtual bool enterHold(Event *event);
 	virtual bool enterRecall(Event *event);
 
@@ -2137,6 +2165,7 @@ protected:
 	bool stateRead(Event *event);
 	bool stateDial(Event *event);
 	bool stateXfer(Event *event);
+	bool stateRefer(Event *event);
 	bool stateHold(Event *event);
 	bool stateRecall(Event *event);
 	bool stateTone(Event *event);
@@ -2148,6 +2177,17 @@ protected:
 	bool stateConnect(Event *event);
 	bool stateReconnect(Event *event);
 
+	/**
+	 * Set call reference pointer.  If previous call is set, then
+	 * it is detached.  Use NULL to clear current call if present.
+	 */
+	void setCall(Call *call, const char *reason = NULL);
+
+	/**
+	 * Make a new call, typically in a child process...
+	 */
+	void makeCall(const char *from);
+
 	/**
 	 * Direct method to post an event to a channel.
 	 *
@@ -2345,6 +2385,7 @@ protected:
 	char var_tid[14];
 	char var_sid[16];
 	char var_pid[16];
+	char var_recall[16];
 	char var_joined[16];
 	char var_rings[4];
 	char var_timeslot[8];
@@ -2559,6 +2600,13 @@ public:
 	inline bool isIdle(void)
 		{return isAvail;};
 
+	/**
+	 * Return if currently referring.
+	 *
+	 * @return true if referring.
+	 */
+	bool isRefer(void);
+
 	/**
 	 * Return state of join.
 	 *
@@ -2566,6 +2614,9 @@ public:
 	 */
 	bool isJoined(void);
 
+	inline bool isHolding(void)
+		{return holding;};
+
 	/**
 	 * Return state disconnecting or idle...
 	 */
