@@ -41,7 +41,7 @@ ScriptInterp(), audio()
 	seq = 0;
 	evseq = 0;
 	tseq = 0;
-	offhook = dtmf = answered = holding = false;
+	offhook = dtmf = answered = holding = referring = false;
 	starttime = 0;
 	type = NONE;
 	seizure = CHILD_RUNNING;
@@ -50,6 +50,7 @@ ScriptInterp(), audio()
 	state.menu = NULL;
 	peer = NULL;
 	connecting = false;
+	call = NULL;
 
 	if(!translator)
 		translator = &langNone;
@@ -68,6 +69,7 @@ ScriptInterp(), audio()
 	strcpy(audio.var_position, "00:00:00.000");
 	strcpy(var_bankid, "0");
 	strcpy(var_pid, "none");
+	strcpy(var_recall, "none");
 	strcpy(var_joined, "none");
 	var_sid[0] = 0;
 	dtmf_digits = NULL;
@@ -99,6 +101,27 @@ BayonneSession::~BayonneSession()
 	finalize();
 }
 
+void BayonneSession::setCall(Call *c, const char *r)
+{
+	if(call && c == call)
+		return;
+
+	if(call)
+		call->detach(r);
+
+	call = c;
+	if(call)
+		call->attach();
+}
+
+void BayonneSession::makeCall(const char *from)
+{
+	if(call)
+		call->detach();
+
+	call = new Call(this, from);
+}
+
 void BayonneSession::incIncomingAttempts(void)
 {
 	incActiveCalls();
@@ -255,6 +278,13 @@ unsigned BayonneSession::getId(void)
 	return (unsigned)timeslot;
 }
 
+bool BayonneSession::isRefer(void)
+{
+        if(state.handler == &BayonneSession::stateRefer)
+                return true;
+        return false;
+}
+
 bool BayonneSession::isJoined(void)
 {
 	if(state.handler == &BayonneSession::stateJoin)
@@ -402,7 +432,20 @@ const char *BayonneSession::getExternal(const char *id)
 			return tmp;
 		}		
 		else if(!stricmp(id, "libext"))
-			return audio.libext;	
+			return audio.libext;
+		else if(!stricmp(id, "status"))
+		{
+			if(holding)
+				return "holding";
+			else if(isJoined())
+				return "joined";
+			else if(isRefer())
+				return "refer";
+			else if(isIdle())
+				return "idle";
+			else
+				return "active";
+		}	
 		else if(!stricmp(id, "line"))
 		{
 			if(holding)
@@ -413,6 +456,8 @@ const char *BayonneSession::getExternal(const char *id)
 		}
 		else if(!stricmp(id, "pid"))
 			return var_pid;
+		else if(!stricmp(id, "recall"))
+			return var_recall;
 		else if(!stricmp(id, "joined") || !stricmp(id, "joinid"))
 			return var_joined;
 		else if(!stricmp(id, "callid") || !stricmp(id, "crn"))
@@ -1336,10 +1381,12 @@ next:
 
 void BayonneSession::makeIdle(void)
 {
-	BayonneSession *parent;
+	BayonneSession *parent, *recall;
 	Symbol *sym;
 	Event event;
 
+	setCall(NULL);
+
 	if(thread)
         {
         	delete thread;
@@ -1370,6 +1417,7 @@ void BayonneSession::makeIdle(void)
         var_sid[0] = 0;
 	holding = false;
 	connecting = false;
+	referring = false;
         sym = mapSymbol("session.digits", MAX_DTMF);
         if(sym)
 	{
@@ -1378,7 +1426,16 @@ void BayonneSession::makeIdle(void)
 		dtmf_digits[0] = 0;
 	}
 
+	recall = getSid(var_recall);
 	parent = getSid(var_pid);
+	if(recall)
+	{
+		memset(&event, 0, sizeof(event));
+		event.peer = this;
+		event.id = DROP_REFER;
+		recall->queEvent(&event);
+		strcpy(var_recall, "none");
+	}
 	if(parent)
 	{
 		memset(&event, 0, sizeof(event));
