@@ -935,7 +935,7 @@ bool BayonneSession::stateConnect(Event *event)
 {
 	Event cancel;
 	BayonneSession *s;
-	bool dtmf, rtn, hangup;
+	bool dtmf, rtn, hangup, refer;
 
         if(enterTone(event))
                 return true;
@@ -1007,9 +1007,11 @@ failed:
 		s = getSid(state.tone.sessionid);
 		dtmf = state.tone.dtmf;
 		hangup = state.tone.hangup;
+		refer = state.tone.refer;
 		state.timeout = state.tone.duration;
 		state.join.dtmf = dtmf;
 		state.join.hangup = hangup;
+		state.join.refer = refer;
 		state.join.dial = NULL;
 		state.join.exit = NULL;
 		state.join.peer = s;
@@ -1154,6 +1156,7 @@ bool BayonneSession::stateRecord(Event *event)
         case STOP_SCRIPT:
         case CALL_DISCONNECT:
         case CALL_FAILURE:
+	case START_REFER:
                 time(&now);
                 if(now - audiotimer > 3 || state.audio.total < 30000)
 		{
@@ -1276,14 +1279,64 @@ bool BayonneSession::stateJoin(Event *event)
 		clrAudio();
 		return true;
         case ENTER_STATE:
+		p = getSid(var_recall);
+		if(p && p != state.join.peer)
+		{
+			memset(&hold_event, 0, sizeof(hold_event));
+			hold_event.peer = this;
+			hold_event.id = DROP_RECALL;
+			p->queEvent(&hold_event);
+		}
                 if(state.timeout && state.timeout != TIMEOUT_INF)
                         startTimer(state.timeout);
+		strcpy(var_recall, "none");
                 peer = state.join.peer;
 		if(state.join.dtmf && !dtmf)
 			dtmf = enableDTMF();
                 setSymbol("script.error", "none");
-		time(&state.join.start);
+		if(call)
+			call->start();
                 return true; 
+	case PEER_REFER:
+		if(!peer || peer != event->peer)
+			return false;
+
+		if(holding)
+		{
+			
+			memset(&hold_event, 0, sizeof(hold_event));
+			hold_event.id = JOIN_RECALL;
+			hold_event.peer = this;
+			event->peer->queEvent(&hold_event);
+			return true;
+		}
+
+		strcpy(var_recall, var_joined);
+		setString(dtmf_digits, MAX_DTMF, event->dialing);
+		if(!scriptEvent("call:transfer"))
+			if(!redirect("connect::transfer"))
+				error("call-transfer");
+		setRunning();
+		referring = false;
+		return false;
+			
+	case START_REFER:
+		if(!holding || !state.join.refer)
+			return false;
+		if(!peer)
+			return false;
+		peer->referring = true;
+		if(peer->holding)
+		{
+			peer->referring = false;
+			return false;
+		}
+		memset(&hold_event, 0, sizeof(hold_event));
+		setString(state.join.digits, sizeof(state.join.digits), event->dialing);
+		state.join.peer = peer;
+		peer = NULL;
+		setState(STATE_REFER);
+		return true;
 	case CALL_HOLD:
 		if(holding)
 			return false;
@@ -1343,12 +1396,9 @@ bool BayonneSession::stateJoin(Event *event)
 		goto part;
 	}
 
-	p = getSid(var_pid);
 	rtn = enterCommon(event);
 	if(!isJoined())
 	{
-		if(p)
-			BayonneBinder::reportBilling(p, this, state.join.start);
 		part(pid);
 		event->id = EXIT_PARTING;
 		enterJoin(event);
@@ -1356,9 +1406,7 @@ bool BayonneSession::stateJoin(Event *event)
 	return rtn;
 
 part:
-	p = getSid(var_pid);
-	if(p)
-			BayonneBinder::reportBilling(p, this, state.join.start);
+	setCall(NULL, msg);
 	if(scriptEvent(msg))
 		state.join.hangup = false;
 	else if(signalScript(SIGNAL_PART))
@@ -1439,6 +1487,62 @@ bool BayonneSession::stateRecall(Event *event)
 	}
 }
 
+bool BayonneSession::stateRefer(Event *event)
+{
+	Event pe;
+
+	if(enterRefer(event))
+		return true;
+
+	switch(event->id)
+	{
+	case ENTER_STATE:
+		memset(&pe, 0, sizeof(pe));
+		pe.id = PEER_REFER;
+		state.join.peer->queEvent(&pe);
+		return true;
+	case JOIN_RECALL:
+		if(event->peer != state.join.peer)
+			return false;
+		strcpy(var_joined, event->peer->var_sid);
+		state.timeout = 0;
+		setState(STATE_JOIN);
+		return true;
+	case DROP_REFER:
+		if(event->peer != state.join.peer)
+			return false;
+		setCall(NULL, "part::disconnect");
+		if(scriptEvent("part:disconnect"))
+			state.join.hangup = false;
+		else if(signalScript(SIGNAL_PART))
+			state.join.hangup = false;
+		else if(!state.join.hangup)
+			error("part-disconnect");
+		if(state.join.hangup)
+			setState(STATE_HANGUP);
+		else
+			setRunning();
+		return true;
+	case DROP_RECALL:
+		if(event->peer != state.join.peer)
+			return false;
+		setCall(NULL, "part:transfer");
+		if(scriptEvent("part:transfer"))
+			state.join.hangup = false;
+		else if(signalScript(SIGNAL_PART))
+			state.join.hangup = false;
+		else if(!state.join.hangup)
+			error("part-transfer");
+		if(state.join.hangup)
+			setState(STATE_HANGUP);
+		else
+			setRunning();
+		return true;
+	default:
+		return enterCommon(event);
+	}
+}
+
 bool BayonneSession::stateXfer(Event *event)
 {
 	if(enterXfer(event))
@@ -2008,7 +2112,6 @@ failure:
 		return true;
 	case AUDIO_DISCONNECT:
 	case AUDIO_RECONNECT:
-		return false;
 	default:
 		return enterCommon(event);
 	}
@@ -2055,17 +2158,30 @@ bool BayonneSession::stateRunning(Event *event)
 			return true;
 		}
 	case TIMER_EXPIRED:
-		if(!holding)
-		{
-			clrAudio();
-			step();
-			check();
-		}
+		if(holding)
+			return true;
+		clrAudio();
+		step();
+		check();
 		if(state.handler == &BayonneSession::stateRunning)
 			startTimer(step_timer);
 		else
 			setSymbol("script.error", "none");
 		return true;
+	case STOP_DISCONNECT:
+	case STOP_SCRIPT:
+	case CALL_DISCONNECT:
+	case LINE_DISCONNECT:
+	case LINE_WINK:
+		if(holding)
+			startTimer(step_timer);
+		return enterCommon(event);
+	case CALL_NOHOLD:
+		if(!holding)
+			return false;
+		startTimer(step_timer);
+		holding = false;
+		return true;
 	default:
 		return enterCommon(event);
 	}
@@ -2184,6 +2300,11 @@ bool BayonneSession::filterPosting(Event *event)
 
 	switch(event->id)
 	{
+	case CALL_HOLD:
+	case CALL_NOHOLD:
+		if(referring)
+			return false;
+		return true;
 	case PART_DISCONNECT:
 	case PART_EXPIRED:
 	case PART_EXITING:
@@ -2192,7 +2313,7 @@ bool BayonneSession::filterPosting(Event *event)
 		return true;
 	case DTMF_KEYUP:
 	case DTMF_KEYDOWN:
-		if(!dtmf || !dtmf_digits)
+		if(!dtmf || !dtmf_digits || holding)
 			return false;
 		digit_count = strlen(dtmf_digits);
 		return true;
@@ -2202,7 +2323,7 @@ bool BayonneSession::filterPosting(Event *event)
 	case CHILD_BUSY:
 	case CHILD_EXPIRED:
 	case CHILD_FAX:
-		if(!image)
+		if(!image || holding)
 			return false;
 		return true;
 	case EXIT_THREAD:
@@ -2257,6 +2378,9 @@ bool BayonneSession::enterCommon(Event *event)
 	case DISABLE_LOGGING:
 		return true;
 
+	case START_REFER:
+		return false;
+
 	case SYSTEM_DOWN:
 		return true;
 
@@ -2300,6 +2424,7 @@ bool BayonneSession::enterCommon(Event *event)
 	case STOP_SCRIPT:
 		if(image)
 		{
+			holding = false;
 			setState(STATE_HANGUP);
 			return true;
 		}
@@ -2372,6 +2497,7 @@ child:
 		if(!offhook)
 			return false;
 
+		holding = false;
 		snprintf(evtmsg, sizeof(evtmsg), "tone:%s", event->tone.name);
 		if(scriptEvent(evtmsg))
 		{
@@ -2382,6 +2508,7 @@ child:
 			goto stop;
 		return false;
 	case LINE_DISCONNECT:
+		holding = false;
 		if(!offhook || !image)
 			return true;
 
@@ -2393,6 +2520,7 @@ child:
 	case STOP_DISCONNECT:
 	case CALL_DISCONNECT:
 stop:
+		holding = false;
 		if(!image)
 			return false;
 
@@ -2409,6 +2537,7 @@ stop:
 		if(!image)
 			return false;
 
+		holding = false;
 		if(!scriptEvent("line:wink"))
 			if(!signalScript(SIGNAL_WINK))
 				return false;
@@ -2419,6 +2548,8 @@ stop:
 		if(!image)
 			return true;
 
+		holding = false;
+
 		if(!scriptEvent("line:pickup"))
 			if(!signalScript(SIGNAL_PICKUP))
 				return false;
@@ -2613,6 +2744,11 @@ bool BayonneSession::enterXfer(Event *event)
 	return false;
 }
 
+bool BayonneSession::enterRefer(Event *event)
+{
+	return false;
+}
+
 bool BayonneSession::enterHold(Event *event)
 {
 	return false;
