@@ -483,9 +483,7 @@ bool Driver::isAvailable(const char *ext)
 			rtn = true;
 		else if(reg->call_limit > reg->traffic->active_calls)
 			rtn = true;
-		if(!gw)
-			rtn = false;
-		else
+		if(regext)
 			switch(reg->traffic->presence)
 			{
 			case SIPTraffic::P_BUSY:
@@ -1967,6 +1965,115 @@ void Driver::run(void)
 				slog.warn("sip: got info for non-existant call");
 				goto killit;
 			}
+			remote_media = NULL;
+			local_media = NULL;
+			conn = NULL;
+			remote_sdp = NULL;
+			local_sdp = NULL;
+			display = NULL;
+			caller = NULL;
+			dialed = NULL;
+			local_uri = NULL;
+			remote_uri = NULL;
+			from = NULL;
+			to = NULL;
+			sdp = NULL;
+			msg = NULL;
+
+			if(MSG_IS_REFER(sevent->request))
+			{
+				session->enterMutex();
+				if(!session->holding)
+				{
+failrefer:
+					session->leaveMutex();
+					eXosip_lock();
+					eXosip_call_build_answer(sevent->tid, 503, &msg);
+					if(msg)
+						eXosip_call_send_answer(sevent->tid, 503, msg);
+					eXosip_unlock();
+					break;
+				}
+				if(!session->isJoined())
+					goto failrefer;
+				if(!session->state.join.refer)
+					goto failrefer;
+				session->leaveMutex();					
+		
+				rtn = 503;
+				osip_message_header_get_byname(sevent->request, "refer-to", 0, &header);
+				if(!header || !header->hvalue)
+					goto norefer;
+				osip_from_to_str(sevent->request->from, &remote_uri);
+				if(!remote_uri)
+					goto norefer;
+				osip_from_init(&from);
+				osip_from_parse(from, remote_uri);
+				if(!from || !from->url)
+					goto norefer; 
+				osip_to_init(&to);
+				osip_to_parse(to, header->hvalue);
+				if(!to || !to->url)
+					goto norefer;
+
+				if(stricmp(to->url->scheme, "sip"))
+				{
+					snprintf(buf, sizeof(buf), "%s:%s@%s:%s",
+						to->url->scheme, to->url->username, to->url->host, to->url->port);
+					goto dorefer;
+				}
+
+				if(!from->url->port)
+					from->url->port = "5060";
+
+				if(!to->url->port)
+					to->url->port = "5060";
+
+				if(!stricmp(from->url->host, to->url->host))
+					if(!stricmp(from->url->port, to->url->port))
+					{
+						setString(buf, sizeof(buf), to->url->username);
+						goto dorefer;
+					}
+						
+				if(to->url->port && stricmp(to->url->port, "5060"))
+					snprintf(buf, sizeof(buf), "sip:%s@%s:%s",
+						to->url->username, to->url->host, to->url->port);
+				else
+					snprintf(buf, sizeof(buf), "sip:%s@%s",
+						to->url->username, to->url->host);
+
+dorefer:
+				memset(&event, 0, sizeof(event));
+				event.id = START_REFER;
+				event.dialing = buf;
+				session->tid = sevent->tid;
+				if(session->postEvent(&event))
+					rtn = 100;
+
+norefer:
+				eXosip_lock();
+				eXosip_call_build_answer(sevent->tid, rtn, &msg);
+				if(msg)
+					eXosip_call_send_answer(sevent->tid, rtn, msg);
+				eXosip_unlock();
+				msg = NULL;
+				goto cleanup;
+			}
+			if(MSG_IS_BYE(sevent->request))
+			{
+				memset(&event, 0, sizeof(event));
+				event.id = STOP_DISCONNECT;
+				session->queEvent(&event);
+				eXosip_lock();
+				msg = NULL;
+				eXosip_call_build_answer(sevent->tid, 200, &msg);
+				if(msg)
+					eXosip_call_send_answer(sevent->tid, 200, msg);
+				eXosip_unlock();
+				msg = NULL;
+				break;
+			}
 			if(!MSG_IS_INFO(sevent->request))
 			{
 				slog.debug("%s: in dialog %s currently unsupported", session->logname, sevent->request->sip_method);
@@ -2256,6 +2363,7 @@ killit:
 				requestOptions(sevent);
 			else if(MSG_IS_PUBLISH(sevent->request))
 			{
+				eXosip_call_get_referto(sevent->did, buf, sizeof(buf));
 				osip_to_to_str(sevent->request->to, &local_uri);
 				if(!local_uri)
 					goto cleanup;
