@@ -302,12 +302,15 @@ protected:
 	Linear peer_buffer;
 	AudioCodec *peer_codec;
 	uint8 dtmf_payload, data_payload;
+	uint8 dtmf_saved, data_saved;
+	Info info_saved;
 	bool dtmf_sipinfo;
-	bool reinvite, reconnecting;
+	bool reinvite, reconnecting, noinvite;
 	volatile bool dtmf_inband;
 	volatile time_t session_timer;
 
-	int cid, did;
+	int cid, did, tid;
+	unsigned long sv;
 
 	bool peerAudio(Encoded encoded);
 
@@ -349,6 +352,8 @@ public:
 
 	bool enterJoin(Event *event);
 
+	bool enterRefer(Event *event);
+
 	bool enterSeize(Event *event);
 
 	bool enterRunning(Event *event);
