@@ -39,11 +39,12 @@ TimerPort()
 	dtmf_inband = Driver::sip.dtmf_inband;
 	dtmf_sipinfo = false;
 
-	rtp_flag = update_pos = reconnecting = false;
+	rtp_flag = update_pos = reconnecting = noinvite = false;
 
 	peer_buffer = NULL;
 	peer_codec = NULL;
 	session_timer = 0;
+	sv = 0;
 }
 
 Session::~Session()
@@ -212,25 +213,25 @@ void Session::redirect(const char *target, const char *encoding, unsigned char d
 	if(dtmf)
 		snprintf(sdp, sizeof(sdp),
 			"v=0\r\n"
-			"o=bayonne 0 0 IN IP4 %s\r\n"
+			"o=bayonne 0 %ld IN IP4 %s\r\n"
 			"s=reinvite\r\n"
 			"c=IN IP4 %s\r\n"
 			"t=0 0\r\n"
 			"m=audio %d RTP/AVP %d %d\r\n"
 			"a=rtpmap:%d %s\r\n"
 			"a=rtpmap:%d telephone-events/8000\r\n",
-			addr, addr, port, data, dtmf,
+			++sv, addr, addr, port, data, dtmf,
 			data, encoding, dtmf);
 	else
 		snprintf(sdp, sizeof(sdp),
 			"v=0\r\n"
-			"o=bayonne 0 0 IN IP4 %s\r\n"
+			"o=bayonne 0 %ld IN IP4 %s\r\n"
 			"s=reinvite\r\n"
 			"c=IN IP4 %s\r\n"
 			"t=0 0\r\n"
 			"m=audio %d RTP/AVP %d\r\n"
 			"a=rtpmap:%d %s\r\n",
-			addr, addr, port, data, data, encoding);
+			++sv, addr, addr, port, data, data, encoding);
 	
 	osip_message_set_body(msg, sdp, strlen(sdp));
 	osip_message_set_content_type(msg, "application/sdp");
@@ -481,6 +482,8 @@ void Session::makeIdle(void)
 	update_pos = false;
 	session_timer = 0;
 	reconnecting = false;
+	noinvite = false;
+	sv = 0;
 
 	if(offhook)
 		sipRelease();
@@ -613,6 +616,42 @@ bool Session::enterJoin(Event *event)
 	}
 }
 
+bool Session::enterRefer(Event *event)
+{
+	osip_message_t *msg = NULL;
+
+	switch(event->id)
+	{
+	case ENTER_STATE:
+		startTimer(3500);
+		return false;
+	case TIMER_EXPIRED:
+		eXosip_lock();
+		eXosip_call_build_answer(tid, 100, &msg);
+		if(msg)
+			eXosip_call_send_answer(tid, 100, msg);
+		eXosip_unlock();
+		startTimer(3500);
+		return true;
+	case DROP_RECALL:
+		eXosip_lock();
+		eXosip_call_build_answer(tid, 200, &msg);
+		if(msg)
+			eXosip_call_send_answer(tid, 200, msg);
+		eXosip_unlock();
+		return false;
+	case DROP_REFER:
+	case JOIN_RECALL:
+		eXosip_lock();
+		eXosip_call_build_answer(tid, 503, &msg);
+		if(msg)
+			eXosip_call_send_answer(tid, 503, msg);
+		eXosip_unlock();
+	default:
+		return false;
+	}
+}
+
 bool Session::enterSeize(Event *event)
 {
 	char buf[128];
@@ -993,7 +1032,6 @@ failure:
 		stopTimer();
 		return true;
 	case STOP_SCRIPT:
-	case STOP_DISCONNECT:
 	case TIMER_EXPIRED:
 	case CANCEL_CHILD:
 		sipHangup();
@@ -1113,6 +1151,11 @@ bool Session::enterReconnect(Event *event)
 	switch(event->id)
 	{
 	case ENTER_RECONNECT:
+		if(noinvite)
+			return false;
+		memcpy(&info_saved, &info, sizeof(info));
+		dtmf_saved = dtmf_payload;
+		data_saved = data_payload;
 		sdp = getSymbol("session.sdp_reconnect");
 		if(!sdp || !*sdp)
 		{
@@ -1177,6 +1220,13 @@ recon:
 		data_payload = sdpPayload();
 		state.timeout = 0;
 		return true;
+	case DIAL_FAILED:
+		memcpy(&info, &info_saved, sizeof(info));
+		dtmf_payload = dtmf_saved;
+		data_payload = data_saved;
+		noinvite = true;
+		startRTP();
+		return true;
 	case CALL_ANSWERED:
 		reconnecting = true;
 		startRTP();
@@ -1256,7 +1306,8 @@ void Session::sipHangup(void)
 {
 	slog.debug("%s: terminating call; cid=%d, did=%d", logname, cid, did);
 	eXosip_lock();
-	eXosip_call_terminate(cid, did);
+	if(cid || did)
+		eXosip_call_terminate(cid, did);
 	eXosip_unlock();
 	stopRTP();
 	offhook = false;
