@@ -546,6 +546,13 @@ retry:
         if(!strnicmp("--", option, 2))
                 ++option;
 
+	if(!strnicmp(option, "-config=", 8))
+	{
+		runtime.setValue("serverconfig", option + 8);
+		++argv;
+		goto retry;
+	}
+
 	if(!strnicmp(option, "-driver=", 8))
 	{
 		keyengine.setValue("driver", option + 8);
@@ -668,6 +675,18 @@ retry:
                 goto retry;
         }
 
+        if(!stricmp(option, "-config"))
+        {
+                ++argv;
+                if(!*argv)
+                {
+                        cerr << "bayonne: -config: missing argument" << endl;
+                        exit(-1);
+                }
+                runtime.setValue("serverconfig", *(argv++));
+                goto retry;
+        }
+
         if(!stricmp(option, "-protocol"))
         {
                 ++argv;
